
namespace eval ::tk::dialog::error {
namespace import -force ::tk::msgcat::*
namespace export bgerror
option add *ErrorDialog.function.text [mc "Save To Log"]  widgetDefault
option add *ErrorDialog.function.command [namespace code SaveToLog]
}

proc ::tk::dialog::error::Return {} {
variable button

.bgerrorDialog.ok configure -state active -relief sunken
update idletasks
after 100
set button 0
}

proc ::tk::dialog::error::Details {} {
set w .bgerrorDialog
set caption [option get $w.function text {}]
set command [option get $w.function command {}]
if { ($caption eq "") || ($command eq "") } {
grid forget $w.function
}
lappend command [.bgerrorDialog.top.info.text get 1.0 end-1c]
$w.function configure -text $caption -command $command
grid $w.top.info - -sticky nsew -padx 3m -pady 3m
}

proc ::tk::dialog::error::SaveToLog {text} {
if { $::tcl_platform(platform) eq "windows" } {
set allFiles *.*
} else {
set allFiles *
}
set types [list	 [list [mc "Log Files"] .log]	 [list [mc "Text Files"] .txt]	 [list [mc "All Files"] $allFiles]  ]
set filename [tk_getSaveFile -title [mc "Select Log File"]  -filetypes $types -defaultextension .log -parent .bgerrorDialog]
if {![string length $filename]} {
return
}
set f [open $filename w]
puts -nonewline $f $text
close $f
}

proc ::tk::dialog::error::Destroy {w} {
if {$w eq ".bgerrorDialog"} {
variable button
set button -1
}
}


proc ::tk::dialog::error::bgerror err {
global errorInfo tcl_platform
variable button

set info $errorInfo

set ret [catch {::tkerror $err} msg];
if {$ret != 1} {return -code $ret $msg}

if {($tcl_platform(platform) eq "macintosh")
|| ([tk windowingsystem] eq "aqua")} {
set ok		[mc Ok]
set messageFont	system
set textRelief	flat
set textHilight	0
} else {
set ok		[mc OK]
set messageFont	{Times -18}
set textRelief	sunken
set textHilight	1
}


set displayedErr ""
set lines 0
foreach line [split $err \n] {
if { [string length $line] > 30 } {
append displayedErr "[string range $line 0 29]..."
break
}
if { $lines > 4 } {
append displayedErr "..."
break
} else {
append displayedErr "${line}\n"
}
incr lines
}

set w .bgerrorDialog
set title [mc "Application Error"]
set text [mc {Error: %1$s} $err]
set buttons [list ok $ok dismiss [mc "Skip Messages"]  function [mc "Details >>"]]


catch {destroy .bgerrorDialog}
toplevel .bgerrorDialog -class ErrorDialog
wm withdraw .bgerrorDialog
wm title .bgerrorDialog $title
wm iconname .bgerrorDialog ErrorDialog
wm protocol .bgerrorDialog WM_DELETE_WINDOW { }

if {($tcl_platform(platform) eq "macintosh")
|| ([tk windowingsystem] eq "aqua")} {
::tk::unsupported::MacWindowStyle style .bgerrorDialog zoomDocProc
}

frame .bgerrorDialog.bot
frame .bgerrorDialog.top
if {[tk windowingsystem] eq "x11"} {
.bgerrorDialog.bot configure -relief raised -bd 1
.bgerrorDialog.top configure -relief raised -bd 1
}
pack .bgerrorDialog.bot -side bottom -fill both
pack .bgerrorDialog.top -side top -fill both -expand 1

set W [frame $w.top.info]
text $W.text				 -bd 2				 -yscrollcommand [list $W.scroll set] -setgrid true			 -width 40				 -height 10				 -state normal			 -relief $textRelief			 -highlightthickness $textHilight	 -wrap char

scrollbar $W.scroll -relief sunken -command [list $W.text yview]
pack $W.scroll -side right -fill y
pack $W.text -side left -expand yes -fill both
$W.text insert 0.0 "$err\n$info"
$W.text mark set insert 0.0
bind $W.text <ButtonPress-1> { focus %W }
$W.text configure -state disabled


set wrapwidth [winfo screenwidth .bgerrorDialog]
set wrapwidth [expr {$wrapwidth-60-[winfo pixels .bgerrorDialog 9m]}]
label .bgerrorDialog.msg -justify left -text $text -font $messageFont  -wraplength $wrapwidth
if {($tcl_platform(platform) eq "macintosh")
|| ([tk windowingsystem] eq "aqua")} {
label .bgerrorDialog.bitmap -bitmap stop
} else {
canvas .bgerrorDialog.bitmap -width 32 -height 32 -highlightthickness 0
.bgerrorDialog.bitmap create oval 0 0 31 31 -fill red -outline black
.bgerrorDialog.bitmap create line 9 9 23 23 -fill white -width 4
.bgerrorDialog.bitmap create line 9 23 23 9 -fill white -width 4
}
grid .bgerrorDialog.bitmap .bgerrorDialog.msg  -in .bgerrorDialog.top	 -row 0			 -padx 3m			 -pady 3m
grid configure	 .bgerrorDialog.msg -sticky nsw -padx {0 3m}
grid rowconfigure	 .bgerrorDialog.top 1 -weight 1
grid columnconfigure .bgerrorDialog.top 1 -weight 1


set i 0
foreach {name caption} $buttons {
button .bgerrorDialog.$name	 -text $caption		 -default normal		 -command [namespace code [list set button $i]]
grid .bgerrorDialog.$name	 -in .bgerrorDialog.bot	 -column $i		 -row 0			 -sticky ew		 -padx 10
grid columnconfigure .bgerrorDialog.bot $i -weight 1
if {($tcl_platform(platform) eq "macintosh")
|| ([tk windowingsystem] eq "aqua")} {
if {($name eq "ok") || ($name eq "dismiss")} {
grid columnconfigure .bgerrorDialog.bot $i -minsize 79
}
}
incr i
}
.bgerrorDialog.ok configure -default active

bind .bgerrorDialog <Return>	[namespace code Return]
bind .bgerrorDialog <Destroy>	[namespace code [list Destroy %W]]
.bgerrorDialog.function configure -command [namespace code Details]


::tk::PlaceWindow .bgerrorDialog


raise .bgerrorDialog
if {$tcl_platform(platform) eq "windows"} {
if {[lindex [wm stackorder .] end] ne ".bgerrorDialog"} {
wm attributes .bgerrorDialog -topmost 1
}
}


::tk::SetFocusGrab .bgerrorDialog .bgerrorDialog.ok


vwait [namespace which -variable button]
set copy $button; # Save a copy...

::tk::RestoreFocusGrab .bgerrorDialog .bgerrorDialog.ok destroy

if {$copy == 1} {
return -code break
}
}

namespace eval :: {
proc bgerror err {}
rename bgerror {}
namespace import ::tk::dialog::error::bgerror
}
