
namespace eval ::tk::dialog {}
namespace eval ::tk::dialog::file {}



proc ::tk::MotifFDialog {type args} {
variable ::tk::Priv
set dataName __tk_filedialog
upvar ::tk::dialog::file::$dataName data

set w [MotifFDialog_Create $dataName $type $args]


::tk::SetFocusGrab $w $data(sEnt)
$data(sEnt) selection range 0 end


vwait ::tk::Priv(selectFilePath)
set result $Priv(selectFilePath)
::tk::RestoreFocusGrab $w $data(sEnt) withdraw

return $result
}


proc ::tk::MotifFDialog_Create {dataName type argList} {
upvar ::tk::dialog::file::$dataName data

MotifFDialog_Config $dataName $type $argList

if {[string equal $data(-parent) .]} {
set w .$dataName
} else {
set w $data(-parent).$dataName
}

if {![winfo exists $w]} {
MotifFDialog_BuildUI $w
} elseif {[string compare [winfo class $w] TkMotifFDialog]} {
destroy $w
MotifFDialog_BuildUI $w
} else {
set data(fEnt) $w.top.f1.ent
set data(dList) $w.top.f2.a.l
set data(fList) $w.top.f2.b.l
set data(sEnt) $w.top.f3.ent
set data(okBtn) $w.bot.ok
set data(filterBtn) $w.bot.filter
set data(cancelBtn) $w.bot.cancel
}
MotifFDialog_SetListMode $w


if {[winfo viewable [winfo toplevel $data(-parent)]] } {
wm transient $w $data(-parent)
}

MotifFDialog_FileTypes $w
MotifFDialog_Update $w


::tk::PlaceWindow $w
wm title $w $data(-title)

return $w
}


proc ::tk::MotifFDialog_FileTypes {w} {
upvar ::tk::dialog::file::[winfo name $w] data

set f $w.top.f3.types
catch {destroy $f}

if {$data(-filetypes) == ""} {
set data(filter) *
return
}

set data(fileType) 0

MotifFDialog_SetFilter $w [lindex $data(-filetypes) $data(fileType)]

if {[llength $data(-filetypes)] == 1} {
return
}

frame $f
set cnt 0
if {$data(-filetypes) != {}} {
foreach type $data(-filetypes) {
set title  [lindex [lindex $type 0] 0]
set filter [lindex $type 1]
radiobutton $f.b$cnt  -text $title  -variable ::tk::dialog::file::[winfo name $w](fileType)  -value $cnt  -command "[list tk::MotifFDialog_SetFilter $w $type]"
pack $f.b$cnt -side left
incr cnt
}
}
$f.b$data(fileType) invoke

pack $f -side bottom -fill both

return
}

proc ::tk::MotifFDialog_SetFilter {w type} {
upvar ::tk::dialog::file::[winfo name $w] data
variable ::tk::Priv

set data(filter) [lindex $type 1]
set Priv(selectFileType) [lindex [lindex $type 0] 0]

MotifFDialog_Update $w
}


proc ::tk::MotifFDialog_Config {dataName type argList} {
upvar ::tk::dialog::file::$dataName data

set data(type) $type

set specs {
{-defaultextension "" "" ""}
{-filetypes "" "" ""}
{-initialdir "" "" ""}
{-initialfile "" "" ""}
{-parent "" "" "."}
{-title "" "" ""}
}
if { [string equal $type "open"] } {
lappend specs {-multiple "" "" "0"}
}

set data(-multiple) 0
if {![info exists data(selectPath)]} {
set data(selectPath) [pwd]
set data(selectFile) ""
}

tclParseConfigSpec ::tk::dialog::file::$dataName $specs "" $argList

if {[string equal $data(-title) ""]} {
if {[string equal $type "open"]} {
if {$data(-multiple) != 0} {
set data(-title) "[mc {Open Multiple Files}]"
} else {
set data(-title) [mc "Open"]
}
} else {
set data(-title) [mc "Save As"]
}
}

if {[string compare $data(-initialdir) ""]} {
if {[file isdirectory $data(-initialdir)]} {
set data(selectPath) [lindex [glob $data(-initialdir)] 0]
} else {
set data(selectPath) [pwd]
}


set old [pwd]
cd $data(selectPath)
set data(selectPath) [pwd]
cd $old
}
set data(selectFile) $data(-initialfile)

set data(-filetypes) [::tk::FDGetFileTypes $data(-filetypes)]

if {![info exists data(filter)]} {
set data(filter) *
}
if {![winfo exists $data(-parent)]} {
error "bad window path name \"$data(-parent)\""
}
}


proc ::tk::MotifFDialog_BuildUI {w} {
set dataName [lindex [split $w .] end]
upvar ::tk::dialog::file::$dataName data

toplevel $w -class TkMotifFDialog
set top [frame $w.top -relief raised -bd 1]
set bot [frame $w.bot -relief raised -bd 1]

pack $w.bot -side bottom -fill x
pack $w.top -side top -expand yes -fill both

set f1 [frame $top.f1]
set f2 [frame $top.f2]
set f3 [frame $top.f3]

pack $f1 -side top    -fill x
pack $f3 -side bottom -fill x
pack $f2 -expand yes -fill both

set f2a [frame $f2.a]
set f2b [frame $f2.b]

grid $f2a -row 0 -column 0 -rowspan 1 -columnspan 1 -padx 4 -pady 4  -sticky news
grid $f2b -row 0 -column 1 -rowspan 1 -columnspan 1 -padx 4 -pady 4  -sticky news
grid rowconfig $f2 0    -minsize 0   -weight 1
grid columnconfig $f2 0 -minsize 0   -weight 1
grid columnconfig $f2 1 -minsize 150 -weight 2

bind [::tk::AmpWidget label $f1.lab -text [mc "Fil&ter:"] -anchor w]  <<AltUnderlined>> [list focus $f1.ent]
entry $f1.ent
pack $f1.lab -side top -fill x -padx 6 -pady 4
pack $f1.ent -side top -fill x -padx 4 -pady 0
set data(fEnt) $f1.ent

set data(dList) [MotifFDialog_MakeSList $w $f2a  [mc "&Directory:"] DList]
set data(fList) [MotifFDialog_MakeSList $w $f2b  [mc "Fi&les:"]     FList]

bind [::tk::AmpWidget label $f3.lab -text [mc "&Selection:"] -anchor w]  <<AltUnderlined>> [list focus $f3.ent]
entry $f3.ent
pack $f3.lab -side top -fill x -padx 6 -pady 0
pack $f3.ent -side top -fill x -padx 4 -pady 4
set data(sEnt) $f3.ent

set maxWidth [::tk::mcmaxamp &OK &Filter &Cancel]
set maxWidth [expr {$maxWidth<6?6:$maxWidth}]
set data(okBtn) [::tk::AmpWidget button $bot.ok -text [mc "&OK"]  -width $maxWidth  -command [list tk::MotifFDialog_OkCmd $w]]
set data(filterBtn) [::tk::AmpWidget button $bot.filter -text [mc "&Filter"]  -width $maxWidth  -command [list tk::MotifFDialog_FilterCmd $w]]
set data(cancelBtn) [::tk::AmpWidget button $bot.cancel -text [mc "&Cancel"]  -width $maxWidth  -command [list tk::MotifFDialog_CancelCmd $w]]

pack $bot.ok $bot.filter $bot.cancel -padx 10 -pady 10 -expand yes  -side left

bind $w <Alt-Key> [list ::tk::AltKeyInDialog $w %A]

bind $data(fEnt) <Return> [list tk::MotifFDialog_ActivateFEnt $w]
bind $data(sEnt) <Return> [list tk::MotifFDialog_ActivateSEnt $w]
bind $w <Escape> [list tk::MotifFDialog_CancelCmd $w]
bind $w.bot <Destroy> {set ::tk::Priv(selectFilePath) {}}

wm protocol $w WM_DELETE_WINDOW [list tk::MotifFDialog_CancelCmd $w]
}

proc ::tk::MotifFDialog_SetListMode {w} {
upvar ::tk::dialog::file::[winfo name $w] data

if {$data(-multiple) != 0} {
set selectmode extended
} else {
set selectmode browse
}
set f $w.top.f2.b
$f.l configure -selectmode $selectmode
}


proc ::tk::MotifFDialog_MakeSList {w f label cmdPrefix} {
bind [::tk::AmpWidget label $f.lab -text $label -anchor w]  <<AltUnderlined>> [list focus $f.l]
listbox $f.l -width 12 -height 5 -exportselection 0 -xscrollcommand [list $f.h set]	-yscrollcommand [list $f.v set]
scrollbar $f.v -orient vertical   -takefocus 0 -command [list $f.l yview]
scrollbar $f.h -orient horizontal -takefocus 0 -command [list $f.l xview]
grid $f.lab -row 0 -column 0 -sticky news -rowspan 1 -columnspan 2  -padx 2 -pady 2
grid $f.l -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
grid $f.v -row 1 -column 1 -rowspan 1 -columnspan 1 -sticky news
grid $f.h -row 2 -column 0 -rowspan 1 -columnspan 1 -sticky news

grid rowconfig    $f 0 -weight 0 -minsize 0
grid rowconfig    $f 1 -weight 1 -minsize 0
grid columnconfig $f 0 -weight 1 -minsize 0

set list $f.l
bind $list <<ListboxSelect>> [list tk::MotifFDialog_Browse$cmdPrefix $w]
bind $list <Double-ButtonRelease-1>  [list tk::MotifFDialog_Activate$cmdPrefix $w]
bind $list <Return>	"tk::MotifFDialog_Browse$cmdPrefix [list $w];  tk::MotifFDialog_Activate$cmdPrefix [list $w]"

bindtags $list [list Listbox $list [winfo toplevel $list] all]
ListBoxKeyAccel_Set $list

return $f.l
}


proc ::tk::MotifFDialog_InterpFilter {w} {
upvar ::tk::dialog::file::[winfo name $w] data

set text [string trim [$data(fEnt) get]]

set badTilde 0
if {[string equal [string index $text 0] ~]} {
set list [file split $text]
set tilde [lindex $list 0]
if {[catch {set tilde [glob $tilde]}]} {
set badTilde 1
} else {
set text [eval file join [concat $tilde [lrange $list 1 end]]]
}
}


set relative 0
if {[string equal [file pathtype $text] "relative"]} {
set relative 1
} elseif {$badTilde} {
set relative 1
}

if {$relative} {
tk_messageBox -icon warning -type ok  -message "\"$text\" must be an absolute pathname"

$data(fEnt) delete 0 end
$data(fEnt) insert 0 [::tk::dialog::file::JoinFile $data(selectPath)  $data(filter)]

return [list $data(selectPath) $data(filter)]
}

set resolved [::tk::dialog::file::JoinFile [file dirname $text] [file tail $text]]

if {[file isdirectory $resolved]} {
set dir $resolved
set fil $data(filter)
} else {
set dir [file dirname $resolved]
set fil [file tail    $resolved]
}

return [list $dir $fil]
}


proc ::tk::MotifFDialog_Update {w} {
upvar ::tk::dialog::file::[winfo name $w] data

$data(fEnt) delete 0 end
$data(fEnt) insert 0  [::tk::dialog::file::JoinFile $data(selectPath) $data(filter)]
$data(sEnt) delete 0 end
$data(sEnt) insert 0 [::tk::dialog::file::JoinFile $data(selectPath)  $data(selectFile)]

MotifFDialog_LoadFiles $w
}


proc ::tk::MotifFDialog_LoadFiles {w} {
upvar ::tk::dialog::file::[winfo name $w] data

$data(dList) delete 0 end
$data(fList) delete 0 end

set appPWD [pwd]
if {[catch {cd $data(selectPath)}]} {
cd $appPWD

$data(dList) insert end ".."
return
}

set top 0
set dlist ""
set flist ""
foreach f [glob -nocomplain .* *] {
if {[file isdir ./$f]} {
lappend dlist $f
} else {
foreach pat $data(filter) {
if {[string match $pat $f]} {
if {[string match .* $f]} {
incr top
}
lappend flist $f
break
}
}
}
}
eval [list $data(dList) insert end] [lsort -dictionary $dlist]
eval [list $data(fList) insert end] [lsort -dictionary $flist]

$data(fList) yview $top

cd $appPWD
}


proc ::tk::MotifFDialog_BrowseDList {w} {
upvar ::tk::dialog::file::[winfo name $w] data

focus $data(dList)
if {[string equal [$data(dList) curselection] ""]} {
return
}
set subdir [$data(dList) get [$data(dList) curselection]]
if {[string equal $subdir ""]} {
return
}

$data(fList) selection clear 0 end

set list [MotifFDialog_InterpFilter $w]
set data(filter) [lindex $list 1]

switch -- $subdir {
. {
set newSpec [::tk::dialog::file::JoinFile $data(selectPath) $data(filter)]
}
.. {
set newSpec [::tk::dialog::file::JoinFile [file dirname $data(selectPath)]  $data(filter)]
}
default {
set newSpec [::tk::dialog::file::JoinFile [::tk::dialog::file::JoinFile  $data(selectPath) $subdir] $data(filter)]
}
}

$data(fEnt) delete 0 end
$data(fEnt) insert 0 $newSpec
}


proc ::tk::MotifFDialog_ActivateDList {w} {
upvar ::tk::dialog::file::[winfo name $w] data

if {[string equal [$data(dList) curselection] ""]} {
return
}
set subdir [$data(dList) get [$data(dList) curselection]]
if {[string equal $subdir ""]} {
return
}

$data(fList) selection clear 0 end

switch -- $subdir {
. {
set newDir $data(selectPath)
}
.. {
set newDir [file dirname $data(selectPath)]
}
default {
set newDir [::tk::dialog::file::JoinFile $data(selectPath) $subdir]
}
}

set data(selectPath) $newDir
MotifFDialog_Update $w

if {[string compare $subdir ..]} {
$data(dList) selection set 0
$data(dList) activate 0
} else {
$data(dList) selection set 1
$data(dList) activate 1
}
}


proc ::tk::MotifFDialog_BrowseFList {w} {
upvar ::tk::dialog::file::[winfo name $w] data

focus $data(fList)
set data(selectFile) ""
foreach item [$data(fList) curselection] {
lappend data(selectFile) [$data(fList) get $item]
}
if {[llength $data(selectFile)] == 0} {
return
}

$data(dList) selection clear 0 end

$data(fEnt) delete 0 end
$data(fEnt) insert 0 [::tk::dialog::file::JoinFile $data(selectPath)  $data(filter)]
$data(fEnt) xview end

$data(sEnt) delete 0 end
if {$data(-multiple) != 0} {
$data(sEnt) insert 0 $data(selectFile)
} else {
$data(sEnt) insert 0 [::tk::dialog::file::JoinFile $data(selectPath)  [lindex $data(selectFile) 0]]
}
$data(sEnt) xview end
}


proc ::tk::MotifFDialog_ActivateFList {w} {
upvar ::tk::dialog::file::[winfo name $w] data

if {[string equal [$data(fList) curselection] ""]} {
return
}
set data(selectFile) [$data(fList) get [$data(fList) curselection]]
if {[string equal $data(selectFile) ""]} {
return
} else {
MotifFDialog_ActivateSEnt $w
}
}


proc ::tk::MotifFDialog_ActivateFEnt {w} {
upvar ::tk::dialog::file::[winfo name $w] data

set list [MotifFDialog_InterpFilter $w]
set data(selectPath) [lindex $list 0]
set data(filter)    [lindex $list 1]

MotifFDialog_Update $w
}


proc ::tk::MotifFDialog_ActivateSEnt {w} {
variable ::tk::Priv
upvar ::tk::dialog::file::[winfo name $w] data

set selectFilePath [string trim [$data(sEnt) get]]

if {[string equal $selectFilePath ""]} {
MotifFDialog_FilterCmd $w
return
}

if {$data(-multiple) == 0} {
set selectFilePath [list $selectFilePath]
}

if {[file isdirectory [lindex $selectFilePath 0]]} {
set data(selectPath) [lindex [glob $selectFilePath] 0]
set data(selectFile) ""
MotifFDialog_Update $w
return
}

set newFileList ""
foreach item $selectFilePath {
if {[string compare [file pathtype $item] "absolute"]} {
set item [file join $data(selectPath) $item]
} elseif {![file exists [file dirname $item]]} {
tk_messageBox -icon warning -type ok  -message [mc {Directory "%1$s" does not exist.}  [file dirname $item]]
return
}

if {![file exists $item]} {
if {[string equal $data(type) open]} {
tk_messageBox -icon warning -type ok  -message [mc {File "%1$s" does not exist.} $item]
return
}
} else {
if {[string equal $data(type) save]} {
set message [format %s%s  [mc "File \"%1\$s\" already exists.\n\n"  $selectFilePath]  [mc {Replace existing file?}]]
set answer [tk_messageBox -icon warning -type yesno  -message $message]
if {[string equal $answer "no"]} {
return
}
}
}

lappend newFileList $item
}

if {$data(-multiple) != 0} {
set Priv(selectFilePath) $newFileList
} else {
set Priv(selectFilePath) [lindex $newFileList 0]
}

set Priv(selectFile)     [file tail    [lindex $newFileList 0]]
set Priv(selectPath)     [file dirname [lindex $newFileList 0]]
}


proc ::tk::MotifFDialog_OkCmd {w} {
upvar ::tk::dialog::file::[winfo name $w] data

MotifFDialog_ActivateSEnt $w
}

proc ::tk::MotifFDialog_FilterCmd {w} {
upvar ::tk::dialog::file::[winfo name $w] data

MotifFDialog_ActivateFEnt $w
}

proc ::tk::MotifFDialog_CancelCmd {w} {
variable ::tk::Priv

set Priv(selectFilePath) ""
set Priv(selectFile)     ""
set Priv(selectPath)     ""
}

proc ::tk::ListBoxKeyAccel_Set {w} {
bind Listbox <Any-KeyPress> ""
bind $w <Destroy> [list tk::ListBoxKeyAccel_Unset $w]
bind $w <Any-KeyPress> [list tk::ListBoxKeyAccel_Key $w %A]
}

proc ::tk::ListBoxKeyAccel_Unset {w} {
variable ::tk::Priv

catch {after cancel $Priv(lbAccel,$w,afterId)}
catch {unset Priv(lbAccel,$w)}
catch {unset Priv(lbAccel,$w,afterId)}
}


proc ::tk::ListBoxKeyAccel_Key {w key} {
variable ::tk::Priv

if { $key == "" } {
return
}
append Priv(lbAccel,$w) $key
ListBoxKeyAccel_Goto $w $Priv(lbAccel,$w)
catch {
after cancel $Priv(lbAccel,$w,afterId)
}
set Priv(lbAccel,$w,afterId) [after 500  [list tk::ListBoxKeyAccel_Reset $w]]
}

proc ::tk::ListBoxKeyAccel_Goto {w string} {
variable ::tk::Priv

set string [string tolower $string]
set end [$w index end]
set theIndex -1

for {set i 0} {$i < $end} {incr i} {
set item [string tolower [$w get $i]]
if {[string compare $string $item] >= 0} {
set theIndex $i
}
if {[string compare $string $item] <= 0} {
set theIndex $i
break
}
}

if {$theIndex >= 0} {
$w selection clear 0 end
$w selection set $theIndex $theIndex
$w activate $theIndex
$w see $theIndex
event generate $w <<ListboxSelect>>
}
}

proc ::tk::ListBoxKeyAccel_Reset {w} {
variable ::tk::Priv

catch {unset Priv(lbAccel,$w)}
}

proc ::tk_getFileType {} {
variable ::tk::Priv

return $Priv(selectFileType)
}

